package com.gxzc.zen.umps.filter

import org.apache.shiro.web.filter.PathMatchingFilter
import org.apache.shiro.web.util.WebUtils
import org.springframework.http.HttpMethod
import org.springframework.http.HttpStatus
import javax.servlet.ServletRequest
import javax.servlet.ServletResponse

/**
 * Cors 的 canon 过滤器
 * @author NorthLan
 * @date 2018/4/25
 * @url https://noahlan.com
 */
class ZenCorsAnonymousFilter : PathMatchingFilter() {

    /**
     * Always returns <code>true</code> allowing unchecked access to the underlying path or resource.
     *
     * @return <code>true</code> always, allowing unchecked access to the underlying path or resource.
     */
    override fun onPreHandle(request: ServletRequest?, response: ServletResponse?, mappedValue: Any?): Boolean {
        val httpRequest = WebUtils.toHttp(request)
        val httpResponse = WebUtils.toHttp(response)
        httpResponse.apply {
            setHeader("Access-control-Allow-Origin", httpRequest.getHeader("Origin"))
            setHeader("Access-Control-Allow-Methods", "GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS, PATCH")
            setHeader("Access-Control-Allow-Credentials", "true")
            setHeader("Access-Control-Allow-Headers", httpRequest.getHeader("Access-Control-Request-Headers"))
            val method = HttpMethod.valueOf(httpRequest.method)
            if (method == HttpMethod.POST || method == HttpMethod.PUT) {
                setHeader("Access-Control-Expose-Headers", "Location")
            } else if (method == HttpMethod.OPTIONS) {
                status = HttpStatus.OK.value()
            }
        }
        return true
    }
}